<?php

namespace Arsenii\WebSockets;

use Arsenii\WebSockets\Facades\Emitter;
use Arsenii\WebSockets\Server;
use Arsenii\WebSockets\Log;

use Closure;

class Connection
{
       
    /**
     * Target
     *
     * @var string
     */
    protected $target = null;
    
    /**
     * Assigned
     *
     * @var string
     */
    protected $Assigned = [];
    
    /**
     * Server
     *
     * @var string
     */
    protected $server = null;

    
    /**
     * Remote address.
     *
     * @var string
     */
    protected $address = '';
    
    /**
     * On change Closure.
     *
     * @var Closure
     */
    protected $onChange = '';
    

    /**
     * Create a new WebSockets Connection instance.
     *
     * @param  string   $id
     * @param  string   $target
     * @param  string   $type
     * @param  string   $server
     * @return void
     */
    public function __construct($target = null, Server $server = null){
        
        $this->target   = $target;
        $this->server   = $server;

        
        Log::info( 'new connection ['. $this->address .']', Log::LEVEL_DEBUG);
        
       
        Emitter::dispatch( 'connecting', $this, '[]' )->when('stoped', function(){

            $this->close();

        })->when('closed', function(){

            Emitter::dispatch( 'connected', $this, '[]' );
        });

    }
  

    /**
     * Close connection.
     *
     * @param mixed $data
     * @param bool $raw
     * @return void
     */
    public function close($data = null, $raw = false){
       
        Emitter::dispatch( 'disconnecting', $this, '[]' )->when('closed', function(){

            $this->destroy();
        });
    }

    /**
     * Destroy connection.
     *
     * @return void
     */
    public function destroy(){
        
        // Unset Connection from Server connections array
        $this->server->unsetConnection( $this->id );

        Emitter::dispatch( 'disconnected', $this, '[]' )->when('closed', function(){
            $this->target->close();
        });

        Log::comment("Destroy was successfully", Log::LEVEL_DEBUG);
    }


    public function __isset($property) {

        if ( property_exists( $this->target, $property) ) {

            return true;
            
        } elseif( isset( $this->Assigned[ $property ] ) ) {
            
            return true;
        }
        
        return false;
    }

    public function __get($property) {

        if ( property_exists( $this->target, $property) ) {

            return $this->target->$property;

        } elseif( isset( $this->Assigned[ $property ] ) ) {

            return $this->Assigned[ $property ];
        }
    }

    public function __set($property, $value) {

        if ( property_exists( $this, $property ) ) {

            $this->$property = $value;

            return;
        }

        if ( property_exists( $this->target, $property ) ) {

            $this->target->$property = $value;

        } else {

            $this->Assigned[ $property ] = $value;
        }

        Log::info("connection changed to [$property => $value]");

        if( is_a( $this->onChange, Closure::class ) ) {

            $callback = $this->onChange;
            $callback( $property, $value );
        }

        return $this;
    }

    public function __call($name, $arguments) {

        if( method_exists( $this, $name ) ) {

           return call_user_func_array([ $this, $name ], $arguments);
        }

        if( method_exists( $this->target, $name ) ) {

           return call_user_func_array([ $this->target, $name ], $arguments);
        }

    }

    public function send($message) {


        if( in_array( gettype($message), ['object', 'array'] ) ){

            if(
                        gettype($message) == 'object'
                    && method_exists( $message, 'toString') 
                ){
                
                $message = $message->toString();

            }else{

                $message = json_encode($message);

            }

        }

        $message = (string) $message;

        $this->target->send($message);
    }
}