<?php

namespace Arsenii\WebSockets;

use Arsenii\WebSockets\Facades\Looper;

use Closure;
use App;

class Emitter
{
    
    /**
     * The array of listenners.
     *
     * @var array
     */
    protected $listeners = [];
    protected $groups    = [];

    protected $types     = [

      'connecting'          => 'onConnecting', //
      'connected'           => 'onConnected', //
      'message-sending'     => 'onMessageSending', //
      'message-sended'      => 'onMessageSended', //
      'message-received'    => 'onMessageReceived', //
      'disconnecting'       => 'onDisconnecting', //
      'disconnected'        => 'onDisconnected', //
      'backlog'             => 'onBacklog', //
    ];

    public function addListeners(){

        eval( 
            "use \Arsenii\WebSockets\Facades\Emitter;".
            "use \Arsenii\WebSockets\Event; ?>".

            file_get_contents( base_path('routes/websockets.php') ) 
        );

        // include base_path('routes/websockets.php');
    }

    public function dispatch( string $type, Connection $connection = null, string $data = null ){

        if(! array_key_exists( $type, $this->types ) )
            return null;

        $event = new Event( $type, $connection, $data );

        Log::info("event created $type");

        Looper::pushEvent($event);

        return $event;
    }    

    public function group( $groups = null, Closure $callback = null ){

        if( !is_null( $callback ) ){

            $this->groups[] = $groups;

            app()->call($callback);

            array_pop( $this->groups );
        }
    }

    public function receiving( String $pattern, $callback ) {

        if( count( $this->groups ) ) {

            $pattern = implode( $this->groups, ' && ') ." && $pattern";
        }

        $listener           = new Listener([
            'pattern' => $pattern,
            'events'  => [
                'message-received' => $callback
            ]
        ]);
        $this->listeners[]  = $listener;

        return $listener;
    }

    public function sending( String $pattern, $callback ) {

        if( count( $this->groups ) ) {

            $pattern = implode( $this->groups, ' && ') ." && $pattern";
        }

        $listener           = new Listener([
            'pattern' => $pattern,
            'events'  => [
                'message-sended' => $callback
            ]
        ]);
        $this->listeners[]  = $listener;

        return $listener;
    }

    public function backlog( String $pattern, $callback ) {

        if( count( $this->groups ) ) {

            $pattern = implode( $this->groups, ' && ') ." && $pattern";
        }

        $listener           = new Listener([
            'pattern' => $pattern,
            'events'  => [
                'backlog' => $callback
            ]
        ]);
        $this->listeners[]  = $listener;

        return $listener;
    }

    public function build( $routes = null ){
        
        $listener           = new Listener( $routes );
        $this->listeners[]  = $listener;

        return $listener;
    }

    public function push( $listenerClass = null ){

        if(
                !is_null( $listenerClass )
            &&  !is_null( ( $instance = App::make($listenerClass) ) )
        ){
            
            $this->listeners[] =  $instance;
        }
    }

    public function callListenersForEvent( Event $event){


        Log::info('call listeners for the event [ '. $event->getRawData() .' ]');
        
        if( in_array( $event->getType(), ['message-sending', 'message-sended', 'message-received', 'backlog'] ) ){
            
            foreach( $this->listeners as $listener ) {
                
                if( $event->match( $listener->pattern() ) ){
                    
                    if( method_exists( $listener, $this->types[ $event->getType() ] ) ){
    
                        Log::info('listener -t 1 [ '. $event->getRawData() .' ]');
                        app()->call( [ $listener, $this->types[ $event->getType() ] ], [ 'event' => $event ] );
                    }
                }
            }
            
        } else {
            
            foreach( $this->listeners as $listener ) {
                
                if( method_exists( $listener, $this->types[ $event->getType() ] ) ){
                    
                    Log::info('listener -t 2 [ '. $event->getRawData() .' ]');
                    app()->call( [ $listener, $this->types[ $event->getType() ] ], [ 'event' => $event ] );
                    
                }
            }
        }        
    }
    
}