<?php

namespace Arsenii\WebSockets;

use \ReflectionFunction;
use \ReflectionMethod;
use \ReflectionObject;
use \ReflectionProperty;

use Closure;

use Arsenii\WebSockets\Log;

final class Event
{


  const SIGN_DESTROY  = 0;
  const SIGN_CHANGE   = 1;
  const SIGN_STOPPED  = 2;
  const SIGN_UNKNOWN  = 3;

  private $connection;
  private $type;
  private $stopped;
  private $message;
  private $_data;

  private $subscribers = [];

  public $pair;
  public $forked = false;

  function __construct( string $type = null, Connection $connection = null, $message = null){
    $this->type       = $type;
    $this->connection = $connection;
    $this->message    = $message;
    $this->_data      = DataResolver::parse($message);

    if( is_null( $this->_data ) )
      $this->_data    = [];

  }

  public function getConnection(){

    return $this->connection;
  }

  public function getData(){

    return $this->_data;
  }

  public function getRawData(){

    return $this->message;
  }

  public function get($flow = null, $default = null){

    return DataResolver::get( $this->_data, $flow, $default );
  }

  public function has( $flow = null ){

    return DataResolver::has( $this->_data, $flow );
  }

  public function getType(){

    return $this->type;
  }

  public function stop(){

    $this->stopped = true;
    exit;
  }

  public function isStopped(){

    return is_null($this->stopped) ? !1 : $this->stopped;
  }

  public function match($pattern = null){

    return DataResolver::match( $this->_data, $pattern );    
  }

  public function send($message = null){

    $this->connection->send($message);
  }

  public function log( string $type = 'info', string $data ){
    
      Log::log( $type, $data );
  }

  public function __get( $property ) {

    $reflection = new ReflectionObject($this);

    if ( $reflection->hasProperty( $property ) ) {
      
      return $this->$property;

    }

  }

  public function sendSign($data = null, $type = self::SIGN_UNKNOWN) {

    if(! $this->pair ) return;
    
    if( $data ) {
        
      $data = serialize([ $type, $data ]) . "\n";

      socket_write($this->pair, $data, strlen($data));  
      Log::comment("sending sign $type = [$data]");
      //dump($this->pair);
    }
  }

  public function when(String $type, Closure $callback) {

    if(! isset( $this->subscribers[ $type ] ) ) {

      $this->subscribers[ $type ] = collect([]);
    }

    $this->subscribers[ $type ]->push($callback);

    return $this;
  }

  public function trigger(String $type) {

    Log::comment("call somthing $type");


    if( isset( $this->subscribers[ $type ] ) ) {

      $this->subscribers[ $type ]->map(function($callback) {

        app()->call( $callback, [ $this ] );

      });
    }
  }

  function __destruct() {
            
    Log::info('event destroyed');

    if( $this->forked ) {

      if( $this->stopped ) {

        $this->sendSign(true, self::SIGN_STOPPED);
        Log::info('sendSign SIGN_STOPPED');

      } else {

        $this->sendSign(true, self::SIGN_DESTROY);
        Log::info('sendSign SIGN_DESTROY');
      }

    }

  }

}
