<?php

namespace Arsenii\WebSockets;


use \ReflectionFunction;
use \ReflectionMethod;

class Listener
{

  private $events = [];
  private $pattern  = '*';

  private static $acceptedEvents = [
    '*',
    'all',
    'connecting',
    'connected',
    'message-sending',
    'message-sended',
    'message-received',
    'disconnecting',
    'disconnected',
    'backlog',
  ];

  function __construct( $routes = null){

    if( is_array( $routes ) ) {

      if( isset( $routes['pattern'] ) ) {

        $this->pattern = (string) $routes['pattern'];
      }

      if( isset( $routes['events'] ) && is_array( $routes['events'] ) ) {

        foreach( $routes['events'] as $type => $callback ) {

          if( in_array($type, self::$acceptedEvents) ) {

            $this->events[ $type ] = $callback;
          }
        }
      }      
    }
  }

  public function onConnecting(Event $event){

    
    if( isset( $this->events['connecting'] ) ) {
      
      self::invoke( $this->events['connecting'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
    
  }

  public function onConnected(Event $event){

    if( isset( $this->events['connected'] ) ) {

      self::invoke( $this->events['connected'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function onMessageReceived(Event $event){

    if( isset( $this->events['message-received'] ) ) {

      self::invoke( $this->events['message-received'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function onMessageSending(Event $event){

    if( isset( $this->events['message-sending'] ) ) {

      self::invoke( $this->events['message-sending'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function onMessageSended(Event $event){

    if( isset( $this->events['message-sended'] ) ) {

      self::invoke( $this->events['message-sended'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function onDisconnecting(Event $event){

    if( isset( $this->events['disconnecting'] ) ) {

      self::invoke( $this->events['disconnecting'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function onDisconnected(Event $event){

    if( isset( $this->events['disconnected'] ) ) {

      self::invoke( $this->events['disconnected'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function onBacklog(Event $event){

    if( isset( $this->events['backlog'] ) ) {

      self::invoke( $this->events['backlog'], $event );
    }

    if( isset( $this->events['all'] ) ) {

      self::invoke( $this->events['all'], $event );
    }

    if( isset( $this->events['*'] ) ) {

      self::invoke( $this->events['*'], $event );
    }
  }

  public function pattern(String $pattern = null) {

    if( $pattern ) {

      $this->pattern = $pattern;

      return $this;
    }

    return $this->pattern;
  }

  public function on(String $type, $callback) {

    if( in_array($type, self::$acceptedEvents) ) {

      $this->events[ $type ] = $callback;
    }
  }

  static public function invoke($path = null, Event $event){

    if( is_string( $path ) ){

      list( $C, $M )    = explode( "@", $path );

      $reflection       = new ReflectionMethod( $C, $M );

    }else{

      $reflection       = new ReflectionFunction( $path );

    }

    $params = [];

    foreach ( $reflection->getParameters() as $P) {

      if(
              $P->getClass()
          &&  $P->getClass()->isInstance( $event )
      ){

          $params[$P->name] = $event;
      }

    }

    if(! count( $params ) ) {

      $params[] = $event;
    }

    // dd($path, $params);

    app()->call( $path, $params );

  }

}
