<?php

namespace Arsenii\WebSockets;

use Arsenii\WebSockets\Facades\Emitter;

use Closure;
use Workerman\Lib\Timer;

class Looper
{
    
    const TIMER_TICK_SEC = 0.5;

    protected $queuedEvents;
    protected $forkedEvents;

    protected function forkEvents() {


        Log::info('queuedEvents [ '. $this->queuedEvents->count() .' ]');
        
        $this->queuedEvents->map(function($event, $index){
            
            $this->queuedEvents->forget($index);

            Log::info('queuedEvents -i [ '. $this->queuedEvents->count() .' ]');

            $this->forkedEvents->push($event);

            $paires = [];
            
            if ( socket_create_pair(AF_UNIX, SOCK_STREAM, 0, $paires) === false ) {
                
                Log::info("socket_create_pair() failed. Reason: ". socket_strerror( socket_last_error() ), Log::LEVEL_DEBUG);
                
                return;
            }
            //dump($paires);

            $pid = pcntl_fork();
            
            if ( $pid == -1 ) {
                
                Log::info("could not fork: ". socket_strerror( socket_last_error() ), Log::LEVEL_DEBUG);
                
            } elseif ( $pid ) {
                
                socket_close( $paires[0] );

                $event->pair = $paires[1];
                //dump($event->pair);

                Log::info('event forked [ '. $event->getRawData() .' ]');
                
            } else {
                
                socket_close( $paires[1] );
                
                $event->pair                    = $paires[0];
                //dump($event->pair);
                $event->forked                  = true;
                $event->connection->onChange    = function( $prop, $value ) use( $event ) {

                    $event->sendSign([$prop, $value], Event::SIGN_CHANGE);
                };


                Log::info('event forked -c [ '. $event->getRawData() .' ]');

                Emitter::callListenersForEvent( $event );
                exit;
            }

        });
    }

    protected function syncStates() {
        
        $toRead = $this->forkedEvents->map(function($event){

            return $event->pair;

        })->all();

        
        if(! count( $toRead ) ) return;
        
        $toWrite    = [];
        $toExcept   = [];
        $readed     = socket_select( $toRead, $toWrite, $toExcept, 0);
        
        
        if ( $readed !== false && $readed > 0 ) {
            
            foreach( $toRead as $pair ) {
                
                $this->forkedEvents->map(function($event, $index) use( $pair ) {
                    
                    
                    if( $event->pair == $pair ) {

                        
                        $data = @socket_read($event->pair, 10000, PHP_NORMAL_READ);

                        Log::info("syncStates read event $index = $data");
                        //dump($event->pair );
                        
                        if( $data == false ) {
                            
                            Log::info("syncStates delete event $index");
                            return $this->forkedEvents->forget($index);
                        }
                        
                        $state = $data ? unserialize( $data ) : null;
                        Log::info("syncStates event data ");
                        //dump($state);

                        if( $state ) {

                            $this->parseState($event, $state);
                        }
                    }

                })->all();
            }
        }

    }

    protected function parseState($event, $state) {

        if( is_array( $state ) && count( $state ) == 2 ) {

            list($sign, $data) = $state;

            if( $sign == Event::SIGN_CHANGE ) {

                if( is_array( $data ) && count( $data ) == 2 ) {

                    list($property, $value) = $data;

                    $event->connection->$property = $value;
                }
            }

            if( $sign == Event::SIGN_STOPPED ) {
                
                $event->trigger('stoped');
            }

            if( $sign == Event::SIGN_DESTROY ) {
                
                $event->trigger('closed');
            }

            if( $sign == Event::SIGN_UNKNOWN ) {

                // UNKNOWN SIGN
            }
        }
    }

    public function start() {

        $this->queuedEvents = collect([]);
        $this->forkedEvents = collect([]);

        Timer::add(
            self::TIMER_TICK_SEC,
            function() {

                $this->forkEvents();
                $this->syncStates();
            }
        );
    }

    public function pushEvent(Event $event) {
        
        $this->queuedEvents->push($event);

        Log::info('event pushed to the looper');
    }
}